<?php

/**
 * Helper function to get action type.
 */
function _constant_contact_action_type() {
  global $user;

	if (!empty($user->uid)) {
		$uid = $user->uid;

    // @TODO: Remove hardcode.
		$allowed = array('Administrator', 'Editor');
		foreach ($user->roles as $role) {
			if (in_array($role, $allowed)) {
        return 'customer';
      }
		}
	}

  return 'contact';
}

/**
 * Implements hook_user_after_update().
 * @see constant_contact_user().
 */
function constant_contact_user_after_update(&$edit, &$account, $category) {
  $subscribe_method = variable_get('cc_register_page_method', CC_REGISTER_PAGE_METHOD);
  $show_format_choice = variable_get('cc_show_format_choice', CC_SHOW_FORMAT_CHOICE);

  // Fixes a bug: http://drupal.org/node/697400
  if ($op === 'after_update') {
    if (!isset($edit['cc_newsletter']) && !isset($account->cc_newsletter) && !count($edit['cc_newsletter_lists']) && !count($account->cc_newsletter_lists)) {
      return;
    }
  }

  $lists = variable_get('cc_lists', '');

  $cc = constant_contact_create_object();

  if (!is_object($cc)) {
    return;
  }

  $fields = variable_get('cc_extra_fields',array());
  $field_mappings = constant_contact_build_field_mappings();

  if ($subscribe_method == 'lists') {
    if ($category == 'account') {
      $lists = $edit['cc_newsletter_lists'];
    }
    else {
      $lists = $account->cc_newsletter_lists;
    }

    $newlists = array();
    foreach($lists as $list_id => $enabled) {
      if (intval($enabled) !== 0) {
        $list = $cc->get_list($list_id);
        $newlists[$list_id] = $list['Name'];
      }
    }

    $lists = $newlists;
  }
  else {
    if ($category == 'account') {
      $edit['cc_newsletter_lists'] = array();
    }
    else {
      $account->cc_newsletter_lists = array();
    }

    $lists = constant_contact_get_lists($cc);
    $enabled_lists = variable_get('cc_lists', array());

    $newlists = array();
    foreach ($lists as $list_id => $list_name) {
      if (in_array($list_id, $enabled_lists)) {
        $newlists[$list_id] = $list_name;
      }
    }

    $lists = $newlists;
  }

  // parse custom fields
  $extra_fields = array();
  foreach($fields as $field) {
    $fieldname = str_replace(' ','', $field);
    if (isset($field_mappings[$fieldname])) {
      $actual_fieldname = $field_mappings[$fieldname];
      // the data we receive here could be from 2 different forms so handle them both
      if ($category == 'account') {
        // if the fieldname has been posted add to our custom CC fields array
        if (isset($edit[$actual_fieldname])) {
          $extra_fields[$fieldname] = $edit[$actual_fieldname];
        }
      }
      else {
        if (isset($account->$actual_fieldname)) {
          $extra_fields[$fieldname] = $account->$actual_fieldname;
        }
      }
    }
  }

  // find contact
  if ($category == 'account') {
    $contact = $cc->query_contacts($edit['mail']);
  }
  else {
    $contact = $cc->query_contacts($account->mail);
  }

  $cc->set_action_type('contact');

  $status = true;
  if ($contact) {
    $email_format = $contact['EmailType'];
    if ($show_format_choice) {
      if ($category == 'account') {
        $email_format = $edit['cc_email_format'];
      }
      else {
        $email_format = $account->cc_email_format;
      }
    }

    $extra_fields['EmailType'] = $email_format;
    $contact = $cc->get_contact($contact['id']);

    // merge contact lists user is already subscribed to
    if ($lists && $contact['lists']) {
      foreach($contact['lists'] as $list_id) {
        if (!isset($lists[$list_id])) {
          $list = $cc->get_list($list_id);
          $lists[$list_id] = $list['Name'];
        }
      }
    }

    if ($category == 'account') {
      if ($subscribe_method == 'checkbox' && $edit['cc_newsletter']) {
        $status = $cc->update_contact($contact['id'], $edit['mail'], array_keys($lists), $extra_fields);
      }
      elseif ($subscribe_method == 'checkbox' && !$edit['cc_newsletter']) {
        $status = $cc->update_contact($contact['id'], $edit['mail'], array(), $extra_fields);
      }
      elseif ($subscribe_method == 'lists' && count($lists) > 0) {
        $status = $cc->update_contact($contact['id'], $edit['mail'], array_keys($lists), $extra_fields);
        $edit['cc_newsletter'] = 1;
      }
      elseif ($subscribe_method == 'lists' && !count($lists)) {
        $status = $cc->update_contact($contact['id'], $edit['mail'], array(), $extra_fields);
      }
    }
    else {
      if ($subscribe_method == 'checkbox' && $account->cc_newsletter) {
        $status = $cc->update_contact($contact['id'], $account->mail, array_keys($lists), $extra_fields);
      }
      elseif ($subscribe_method == 'checkbox' && !$account->cc_newsletter) {
        $status = $cc->update_contact($contact['id'], $account->mail, array(), $extra_fields);
      }
      elseif ($subscribe_method == 'lists' && count($lists) > 0) {
        $status = $cc->update_contact($contact['id'], $account->mail, array_keys($lists), $extra_fields);
        $account->cc_newsletter = 1;
      }
      elseif ($subscribe_method == 'lists' && !count($lists)) {
        $status = $cc->update_contact($contact['id'], $account->mail, array(), $extra_fields);
      }
    }
  }
  elseif ($subscribe_method == 'checkbox' && $edit['cc_newsletter']) {
    $status = $cc->create_contact($edit['mail'], array_keys($lists), $extra_fields);
  }
  elseif ($subscribe_method == 'lists' && count($lists) > 0) {
    $status = $cc->create_contact($edit['mail'], array_keys($lists), $extra_fields);
    $edit['cc_newsletter'] = 1;
  }

  if (!$status) {
    drupal_set_message(
      t(constant_contact_display_last_error($cc->http_response_code)),
      'error'
    );
  }
}

/**
 * Implements hook_user_form().
 * @see constant_contact_user().
 */
function constant_contact_user_form(&$edit, &$account, $category) {
  $lists = variable_get('cc_lists', '');
  $default_subscribe_format = variable_get('cc_subscribe_format', CC_SUBSCRIBE_FORMAT);

  // alter the user edit form
  $selected_lists = array();

  $cc = constant_contact_create_object();

  if (!is_object($cc)) {
    return;
  }

  // get contact and selected lists
  $contact = $cc->query_contacts($account->mail);
  $contact = $cc->get_contact($contact['id']);

  if ($contact['lists']) {
    $selected_lists = $contact['lists'];
  }

  $form['constant_contact'] = array(
    '#type' => 'fieldset',
    '#title' => t('Newsletter'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#tree' => FALSE,
    '#description' => t('Manage your newsletter subscription'),
  );

  // check with the returned API data if they are active or not
  // if syncing is disabled this will mean their status is reflected correctly
  if ($contact) {
    if ($contact['Status'] == 'Active') {
      $account->cc_newsletter = 1;
    }
    else {
      $account->cc_newsletter = 0;
    }

    $default_subscribe_format = $contact['EmailType'];
  }

  if ('lists' === variable_get('cc_register_page_method', CC_REGISTER_PAGE_METHOD)) {
    $exclude_lists = variable_get('cc_lists',array());

    $options = array();
    foreach (constant_contact_get_lists($cc) as $list_id => $list_name) {
      if (!in_array($list_id, $exclude_lists)) {
        $options[$list_id] = $list_name;
      }
    }

    if (!empty($options)) {
      $list_format = variable_get('cc_list_selection_format', CC_LIST_SELECTION_FORMAT);

      if ($list_format == 'select') {
        $field_type = 'select';
      }
      else {
        $field_type = 'checkboxes';
      }

      $form['constant_contact']['cc_newsletter_lists'] = array(
        '#type' => $field_type,
        '#title' => variable_get('cc_signup_title', CC_SIGNUP_TITLE),
        '#description' => variable_get('cc_signup_description', CC_SIGNUP_DESCRIPTION),
        '#options' => $options,
        '#default_value' => $selected_lists,
        '#weight' => 11,
      );

      if ($list_format === 'select') {
        $field_size = ($options && count($options) > 25) ? 25 : count($options);
        $form['constant_contact']['cc_newsletter_lists']['#multiple'] = true;
        $form['constant_contact']['cc_newsletter_lists']['#size'] = count($options);
      }
    }
  }
  else {
    $form['constant_contact']['cc_newsletter'] = array(
      '#type' => 'checkbox',
      '#title' => variable_get('cc_signup_title', CC_SIGNUP_TITLE),
      '#description' => variable_get('cc_signup_description', CC_SIGNUP_DESCRIPTION),
      '#weight' => 10,
      '#default_value' => $account->cc_newsletter,
    );
  }

  if (variable_get('cc_show_format_choice', CC_SHOW_FORMAT_CHOICE)) {
    $form['constant_contact']['cc_email_format'] = array(
      '#type' => 'radios',
      '#title' => t('Email Format'),
      '#description' => 'You can receive emails in Text or HTML format',
      '#weight' => 12,
      '#default_value' => $default_subscribe_format,
      '#options' => array('Text' => 'Text', 'HTML' => 'HTML'),
    );
  }

  return $form;
}

/**
 * Implements hook_user_insert().
 * @see constant_contact_user().
 */
function constant_contact_user_insert(&$edit, &$account, $category) {
  // Fixes a bug: http://drupal.org/node/697400
  if (
    !isset($edit['cc_newsletter'])
    && !isset($account->cc_newsletter)
    && !count($edit['cc_newsletter_lists'])
    && !count($account->cc_newsletter_lists)
  ) {
    return;
  }

  $cc = constant_contact_create_object();
  $action_type = _constant_contact_action_type();

  if (!is_object($cc)) {
    return;
  }

  $show_format_choice = variable_get('cc_show_format_choice', CC_SHOW_FORMAT_CHOICE);
  $default_subscribe_format = variable_get('cc_subscribe_format',CC_SUBSCRIBE_FORMAT);
  $fields = variable_get('cc_extra_fields',array());
  $field_mappings = constant_contact_build_field_mappings();

  if ('checkbox' === variable_get('cc_register_page_method', CC_REGISTER_PAGE_METHOD)) {
    if (!$edit['cc_newsletter']) {
      return;
    }

    $lists = variable_get('cc_lists', array());
  }
  else {
    $lists = $edit['cc_newsletter_lists'];

    $newlists = array();
    foreach ($lists as $list_id => $enabled) {
      if (intval($enabled) !== 0) {
        $list = $cc->get_list($list_id);
        $newlists[$list_id] = $list['Name'];
      }
    }
    $lists = $newlists;
  }

  if (empty($lists)) {
    return;
  }

  $edit['cc_newsletter'] = 1; # they may not have this flag set but may be subscribed

  // parse custom fields
  $extra_fields = array();
  if (is_array($fields)) {
    foreach ($fields as $field) {
      $fieldname = str_replace(' ','', $field);
      if (isset($field_mappings[$fieldname]) && isset($_POST[$field_mappings[$fieldname]])) {
        $extra_fields[$fieldname] = $_POST[$field_mappings[$fieldname]];
      }
    }
  }

  // find contact
  $contact = $cc->query_contacts($edit['mail']);
  $cc->set_action_type($action_type);

  $email_format = ($contact) ? $contact['EmailType'] : $default_subscribe_format;
  if ($show_format_choice && isset($edit['cc_email_format'])) {
    $email_format = $edit['cc_email_format'];
  }

  $extra_fields['EmailType'] = $email_format;

  if ($contact) {
    $contact = $cc->get_contact($contact['id']);

    // merge contact lists user is already subscribed to
    if ($lists && $contact['lists']) {
      foreach ($contact['lists'] as $list_id) {
        if (!isset($lists[$list_id])) {
          $list = $cc->get_list($list_id);
          $lists[$list_id] = $list['Name'];
        }
      }
    }

    $status = $cc->update_contact($contact['id'], $edit['mail'], array_keys($lists), $extra_fields);
  }
  else {
    $status = $cc->create_contact($edit['mail'], array_keys($lists), $extra_fields);
  }

  if (!$status) {
    drupal_set_message(t(constant_contact_display_last_error($cc->http_response_code)), 'error');
  }
}

/**
 * Implements hook_user_delete().
 * @see constant_contact_user().
 */
function constant_contact_user_delete($account) {
  $cc = constant_contact_create_object();

  if (is_object($cc) && $account->cc_newsletter) {
    // find contact
    if ($contact = $cc->query_contacts($account->mail)) {
      // IMMPORTANT: tells CC that the contact or customer made this action
      $cc->set_action_type(_constant_contact_action_type());
      $cc->delete_contact($contact['id']);
    }
  }
}
